/* OGMRipNeroAac - A plugin for OGMRip adding support for NeroAac
 * Copyright (C) 2004-2009 Olivier Rolland <billl@users.sourceforge.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <ogmrip.h>
#include <ogmjob.h>

#include <ogmrip-mplayer.h>
#include <ogmrip-plugin.h>

#include <glib/gi18n-lib.h>

typedef struct _OGMRipNeroAac OGMRipNeroAac;
typedef struct _OGMRipNeroAacClass OGMRipNeroAacClass;

struct _OGMRipNeroAac
{
  OGMRipAudioCodec parent_instance;
};

struct _OGMRipNeroAacClass
{
  OGMRipAudioCodecClass parent_class;
};

static gint ogmrip_nero_aac_run (OGMJobSpawn *spawn);

static void
ogmrip_nero_aac_class_init (OGMRipNeroAacClass *klass)
{
  OGMJobSpawnClass *spawn_class = OGMJOB_SPAWN_CLASS (klass);

  /*
   * Override the run () function
   */
  spawn_class->run = ogmrip_nero_aac_run;
}

static void
ogmrip_nero_aac_init (OGMRipNeroAac *nouveau)
{
}

G_DEFINE_TYPE (OGMRipNeroAac, ogmrip_nero_aac, OGMRIP_TYPE_AUDIO_CODEC)

static gchar **
ogmrip_wav_command (OGMRipAudioCodec *audio, const gchar *output)
{
  GPtrArray *argv;

  /*
   * A function to create the WAV/PCM command lines is available
   * in libogmrip-mplayer
   */
  argv = ogmrip_mplayer_wav_command (audio, TRUE, output);
  g_ptr_array_add (argv, NULL);

  return (gchar **) g_ptr_array_free (argv, FALSE);
}

static gchar **
ogmrip_aac_command (OGMRipAudioCodec *audio, const gchar *input)
{
  GPtrArray *argv;
  const gchar *output;
  gint quality;

  g_return_val_if_fail (OGMRIP_IS_AUDIO_CODEC (audio), NULL);
  g_return_val_if_fail (input != NULL, NULL);

  output = ogmrip_codec_get_output (OGMRIP_CODEC (audio));
  g_return_val_if_fail (output != NULL, NULL);

  /*
   * In OGMRip, the quality is ranged from 0 to 10
   */
  quality = ogmrip_audio_codec_get_quality (audio);

  argv = g_ptr_array_new ();

  /*
   * The name of the executable
   */
  g_ptr_array_add (argv, g_strdup ("neroAacEnc"));
  g_ptr_array_add (argv, g_strdup ("-ignorelength"));

  /*
   * The quality in neroAacEnc is a floating-point number in 0..1 range
   */
  g_ptr_array_add (argv, g_strdup ("-q"));
  g_ptr_array_add (argv,
      g_strdup_printf ("%d.%d", quality / 10, quality % 10));

  /*
   * The input file; the FIFO actually
   */
  g_ptr_array_add (argv, g_strdup ("-if"));
  g_ptr_array_add (argv, g_strdup (input));

  /*
   * The output file
   */
  g_ptr_array_add (argv, g_strdup ("-of"));
  g_ptr_array_add (argv, g_strdup (output));

  g_ptr_array_add (argv, NULL);

  return (gchar **) g_ptr_array_free (argv, FALSE);
}

static gint
ogmrip_nero_aac_run (OGMJobSpawn *spawn)
{
  OGMJobSpawn *pipeline;
  OGMJobSpawn *child;

  gchar *fifo, **argv;
  gint result = OGMJOB_RESULT_ERROR;

  /*
   * Create the FIFO
   */
  fifo = ogmrip_fs_mkftemp ("fifo.XXXXXX", NULL);

  /*
   * Create the pipeline
   */
  pipeline = ogmjob_pipeline_new ();

  /*
   * Add the pipeline in the container
   */
  ogmjob_container_add (OGMJOB_CONTAINER (spawn), pipeline);

  /*
   * Unreference the pipeline
   */
  g_object_unref (pipeline);

  /*
   * Create the command line to extract in WAV
   */
  argv = ogmrip_wav_command (OGMRIP_AUDIO_CODEC (spawn), fifo);
  if (argv)
  {
    /*
     * Create the process
     */
    child = ogmjob_exec_newv (argv);

    /*
     * Add a function to parse the output of the process and report the
     * progression. This function is defined in libogmrip-mplayer
     */
    ogmjob_exec_add_watch_full (OGMJOB_EXEC (child),
        (OGMJobWatch) ogmrip_mplayer_wav_watch, spawn, TRUE, FALSE, FALSE);

    /*
     * Add the process in the pipeline
     */
    ogmjob_container_add (OGMJOB_CONTAINER (pipeline), child);

    /*
     * Unreference the process
     */
    g_object_unref (child);

    /*
     * Create the command line to convert the WAV stream to AAC
     */
    argv = ogmrip_aac_command (OGMRIP_AUDIO_CODEC (spawn), fifo);
    if (argv)
    {
      /*
       * Create the process
       */
      child = ogmjob_exec_newv (argv);

      /*
       * Add the process in the pipeline
       */
      ogmjob_container_add (OGMJOB_CONTAINER (pipeline), child);

      /*
       * Unreference the process
       */
      g_object_unref (child);

      /*
       * Call the run() function of the parent object. This is a
       * blocking execution
       */
      result = OGMJOB_SPAWN_CLASS (ogmrip_nero_aac_parent_class)->run (spawn);
    }
  }

  /*
   * After then encoding, remove the pipeline from the container
   */
  ogmjob_container_remove (OGMJOB_CONTAINER (spawn), pipeline);

  /*
   * Remove the FIFO
   */
  ogmrip_fs_unref (fifo, TRUE);

  return result;
}

static OGMRipAudioPlugin nero_aac_plugin =
{
  NULL,
  G_TYPE_NONE,
  "nero-aac",
  "Nero AAC",
  OGMRIP_FORMAT_AAC
};

OGMRipAudioPlugin *
ogmrip_init_plugin (GError **error)
{
  gchar *fullname;

  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

#ifdef ENABLE_NLS
  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
  textdomain (GETTEXT_PACKAGE);
#endif /* ENABLE_NLS */

  if (!ogmrip_check_mplayer ())
    return NULL;

  fullname = g_find_program_in_path ("neroAacEnc");
  if (!fullname)
  {
    g_set_error (error, OGMRIP_PLUGIN_ERROR, OGMRIP_PLUGIN_ERROR_REQ, _("neroAacEnc is missing"));
    return NULL;
  }
  g_free (fullname);

  nero_aac_plugin.type = ogmrip_nero_aac_get_type ();

  return &nero_aac_plugin;
}

